<?php

/**
 * This file is only loaded if we're on the admin dashboard
 */


// security - stop if file isn't accessed via WP
if (!defined('ABSPATH'))
	exit;


/**
 * Class to interface with the Wordpress Settings API
 *
 * https://codex.wordpress.org/Settings_API
 */
class feedbackcompany_admin_settings
{
	// holds our settings
	private $options;

	/**
	 * these variables can be set to true in the object to perform certain actions
	 * during destruction time (on Wordpress shutdown)
	 */
	private $updated_oauth = false;
	private $updated_widgetmain = false;
	private $updated_widgetproductsummary = false;
	private $updated_widgetproductextended = false;
	private $updated_widgetproductextended_toggle_element = false;
	private $before_product_reviews_enabled_check;

	/**
	 * Constructor initializes everything we need for the admin dashboard
	 */
	public function __construct()
	{
		// initialize the API once, so an API call is made
		// this call is also necessary for succesful upgrade, as this object converts old-style options to new
		feedbackcompany_api_wp();

		// add configuration page to admin menu
		add_action('admin_menu', array($this, 'add_plugin_page'));
		// initialize configuration options
		add_action('admin_init', array($this, 'page_init'));
		// load admin javascript
		add_action('admin_enqueue_scripts', array($this, 'scripts'));
		// load admin css
		add_action('admin_enqueue_scripts', array($this, 'styles'));

		// special action for handling TXT file download of the error log
		if (isset($_GET['page']) && $_GET['page'] == 'feedbackcompany' && isset($_GET['downloaderrorlog']))
		{
			add_action('admin_init', array($this, 'downloaderrorlog'));
		}
		if (isset($_GET['page']) && $_GET['page'] == 'feedbackcompany' && isset($_GET['deleteerrorlog']))
		{
			add_action('admin_init', array($this, 'deleteerrorlog'));
		}
	}

	/**
	 * Function adds our settings page to the Wordpress menu
	 */
	public function add_plugin_page()
	{
		// this page will be under "Settings"
		add_options_page(
			'Feedback Company',
			'Feedback Company',
			'manage_options',
			'feedbackcompany',
			array($this, 'output_adminpage')
		);
	}

	/**
	 * Function loads the javascript for our settings page
	 */
	function scripts($hook)
	{
		if ($hook != 'settings_page_feedbackcompany')
			return false;

		wp_enqueue_script('jquery');
		wp_enqueue_script('jquery-ui');
		wp_enqueue_script('jquery-ui-tooltip');

		wp_enqueue_script('feedbackcompany_admin_javascript', plugins_url('js/admin.js', __FILE__));
		wp_localize_script('feedbackcompany_admin_javascript', 'feedbackcompany_admin_javascript',
			array('plugins_url' => plugins_url('images/widgets-preview/', __FILE__))
		);
	}

	/**
	 * Function loads the CSS for our settings page
	 */
	function styles($hook)
	{
		if ($hook != 'settings_page_feedbackcompany')
			return false;

		wp_enqueue_style('jquery-ui-style', '//code.jquery.com/ui/1.11.4/themes/smoothness/jquery-ui.css');
		wp_enqueue_style('feedbackcompany_admin_stylesheet', plugins_url('css/admin.css', __FILE__));
	}

	/**
	 * Helper function for setting default configuration options
	 *
	 * if $key has no value, then $value is set
	 *
	 * @param string $key - option key
	 * @param string $value - option default value
	 */
	public function option_set_default($key, $value)
	{
		if (get_option($key) === false)
			update_option($key, $value);
	}

	/**
	 * This function defines all settings for our plugin, and loads the defaults and registers callbacks for output
	 */
	public function page_init()
	{
		// set default options if not defined
		$this->option_set_default('feedbackcompany_wordpressmultilanguage', 'all');
		$this->option_set_default('feedbackcompany_oauth_client_id', '');
		$this->option_set_default('feedbackcompany_oauth_client_secret', '');
		$this->option_set_default('feedbackcompany_invitation_orderstatus', 'wc-completed');
		$this->option_set_default('feedbackcompany_invitation_delay', '7');
		$this->option_set_default('feedbackcompany_invitation_delay_unit', 'days');
		$this->option_set_default('feedbackcompany_invitation_reminder', '14');
		$this->option_set_default('feedbackcompany_invitation_reminder_unit', 'days');
		$this->option_set_default('feedbackcompany_merchantreviewswidget_size', 'small');
		$this->option_set_default('feedbackcompany_merchantreviewswidget_amount', 0);
		$this->option_set_default('feedbackcompany_productreviewsextendedwidget_displaytype', 'sidebar');
		$this->option_set_default('feedbackcompany_productreviews_enabled', false);

		// name of our page
		$page = 'feedbackcompany_settings';

		// define settings fields
		$fields = array();
		$fields['section_oauth'] = array('section', 'Authentication Settings');
		$fields['feedbackcompany_oauth_client_id'] = array('text', 'Client ID');
		$fields['feedbackcompany_oauth_client_secret'] = array('text', 'Client secret');

		// WPML settings only if WPML is enabled
		if (feedbackcompany_wordpressmultilanguage_enabled())
		{
			$fields['section_wpml'] = array('section', 'Choose your active language');
			$fields['feedbackcompany_wordpressmultilanguage'] = array('wpml', 'Language');
		}

		// woocommerce settings only if woocommerce is enabled
		if (feedbackcompany_woocommerce_enabled())
		{
			$fields['section_woocommerce'] = array('section', 'WooCommerce Settings');
			$fields['feedbackcompany_invitation_orderstatus'] = array('orderstatus', 'Order status');
			$fields['feedbackcompany_invitation_delay'] = array('timeframe', 'Invitation delay');
			$fields['feedbackcompany_invitation_reminder'] = array('timeframe', 'Reminder delay');
			if (get_option('feedbackcompany_productreviews_enabled'))
			{
				$fields['section_productreviewswidget'] = array('section', 'Product reviews widget');
				$fields['feedbackcompany_productreviewsextendedwidget_displaytype'] = array('displaytype', 'Product page widget');
				$fields['feedbackcompany_productreviewsextendedwidget_toggle_element'] = array('text', 'Widget toggle element');
			}
			else
			{
				$fields['section_productreview'] = array('section', 'Product reviews widget disabled');
			}

		}
		$fields['section_merchantreviewswidget'] = array('section', 'Merchant reviews widget');
		$fields['feedbackcompany_merchantreviewswidget_size'] = array('size', 'Size');
		$fields['feedbackcompany_merchantreviewswidget_amount'] = array('amount', 'Amount of reviews');
		$fields['feedbackcompany_merchantreviewswidget_shortcode'] = array('shortcode', 'Shortcode', 'feedback_company_merchant_reviews_widget');


		$fields['section_widgetspreview'] = array('section', 'Widgets preview');
		// holds current section id while we loop over settings
		$setting_section_id = null;

		// loop over all form fields and sections
		foreach ($fields as $name => $field)
		{
			// if this entry is a section, add it with the appropriate callback
			if ($field[0] == 'section')
			{
				add_settings_section(
					$name,
					$field[1],
					array($this, 'section_output'),
					$page
				);
				$setting_section_id = $name;
			}
			// if this entry is a form field, register the setting and add the field with the appropriate callback
			else
			{
				// for timeframe fields, also register the _unit setting
				if ($field[0] == 'timeframe')
					register_setting('feedbackcompany_option_group', $name.'_unit');

				register_setting(
					'feedbackcompany_option_group',
					$name,
					array($this, 'sanitize')
				);

				add_settings_field(
					$name,
					$field[1],
					array($this, 'field_output_'.$field[0]),
					$page,
					$setting_section_id,
					array('name' => $name, 'desc' => isset($field[2]) ? $field[2] : '')
				);
			}
		}
	}

	/**
	 * This function is called on form save to sanitize data
	 *
	 * we're not using this, so return the original data
	 */
	public function sanitize($input)
	{
		return $input;
	}

	/**
	 * Function to return all API errors
	 *
	 * This is a helper function for below, but is also called later to find out
	 * if we need to display a download button or not
	 */
	public function errorlog()
	{
		global $wpdb;
		$table_name = $wpdb->prefix.'feedbackcompany_errorlog';
		if ($wpdb->get_var("SHOW TABLES LIKE '$table_name'") != $table_name)
			return array();

		return $wpdb->get_results("SELECT * FROM `$table_name`");
	}

	/**
	 * Function is called when the user has chosen to download the error log
	 *
	 * This function completely overrides default Wordpress output
	 * and instead serves a TXT file with our API errors
	 */
	public function downloaderrorlog()
	{

		// send error log as TXT file to browser
		header("Content-type: application/x-msdownload");
		header("Content-Disposition: attachment; filename=feedbackcompany-errorlog.txt");
		header("Pragma: no-cache");
		header("Expires: 0");
		$errorlog = $this->errorlog();
		foreach ($errorlog as $error)
		{
			$eol = "\r\n";
			echo $error->timestamp.$eol
				. $error->url.$eol
				. 'Call: '.$error->call.$eol
				. 'Response: '.$error->response.$eol.$eol;
		}

		// exit here, to suppress normal Wordpress output
		exit();
	}

	/**
	 * Function is called when the user has chosen to delete the error log
	 * Deletes content from the errorlog table
	 */
	public function deleteerrorlog()
	{
		global $wpdb;
		$table_name = $wpdb->prefix.'feedbackcompany_errorlog';
		$wpdb->query("TRUNCATE TABLE $table_name");
		wp_redirect(admin_url('options-general.php?page=feedbackcompany'));
		exit();
	}

	/**
	 * This function outputs the requested admin page, current or legacy
	 *
	 * This code can be dropped once legacy support is
	 */
	public function output_adminpage()
	{
		if (isset($_GET['legacywoocommerce']))
			return $this->output_adminpage_legacywoocommerce();
		if (isset($_GET['legacywidgets']))
			return $this->output_adminpage_legacywidgets();

		return $this->output_adminpage_current();
	}

	/**
	 * Output function for the settings page of the current version of this plugin
	 */
	public function output_adminpage_current()
	{
		echo '<div class="wrap">';
		echo '<img align="right" style="width: 30vw; max-width: 300px; max-height: 28px; margin-top: 10px; margin-bottom: -100%;" src="'.plugins_url('/images/feedbackcompany-logo.svg', __FILE__).'">';
		echo '<h1>Feedback Company</h1>';

		// admin notice if there's no oauth details entered
		if (!get_option('feedbackcompany_oauth_client_id') || !get_option('feedbackcompany_oauth_client_secret'))
		{
			echo '<div class="updated notice"><p>';
			echo 'This plugin requires a Feedback Company account. Please visit <a target="_blank" href="http://www.feedbackcompany.com/">www.feedbackcompany.com</a> to register and enter the OAuth details you receive below.';
			echo '</p></div>';
		}
		// admin error if there is oauth details entered, but we weren't able to get an access token
		elseif (!feedbackcompany_api_wp()->ext->get_option('feedbackcompany_access_token'))
		{
			echo '<div class="error notice"><p>';
			echo 'Wordpress was unable to connect to Feedback Company using the OAuth client & secret below. Please verify your settings or contact us if you think this is in error.';
			echo '</p></div>';
		}

		echo '<form onsubmit="return feedbackcompany_validateform();" method="post" action="options.php">';

		// This prints out all settings fields and calls the appropriate callbacks
		settings_fields('feedbackcompany_option_group');
		do_settings_sections('feedbackcompany_settings');

		// buttons for submit and downloading error logs
		echo '<p class="submit">';
		submit_button(null, 'primary', 'submit', false);
		if (count($this->errorlog()))
		{
			echo '<a class="button-secondary" href="?page=feedbackcompany&downloaderrorlog">Download log file</a>';
			echo '<a class="button-secondary" href="?page=feedbackcompany&deleteerrorlog">Delete log file</a>';
		}
		echo '</p>';

		echo '</form>';

		// buttons for legacy config, if legacy widgets or woocommerce is enabled

		if (feedbackcompany_legacywidgets_enabled() || feedbackcompany_legacywoocommerce_enabled())
		{
			echo '<h1>Legacy configuration for 1.x versions of this plugin</h1>';
			echo '<p class="max600px">';
			echo 'Your Wordpress installation contains configuration for an older version of this plugin.  The 1.x versions are no longer developed, '
				. 'but functionality is integrated into the current version of this plugin for backwards compatibility. '
				. '<br><br>Everything still works like it did (for now) and configuration is available '
				. 'via the links below. Support for these versions will be dropped in the future however, '
				. 'so please migrate to the new version using the configuration screen above when you can.';
			echo '</p>';
			if (feedbackcompany_legacywidgets_enabled())
				echo '<a class="button-secondary" href="?page=feedbackcompany&legacywidgets">Legacy Review Widgets configuration</a> &nbsp; ';
			if (feedbackcompany_legacywoocommerce_enabled())
				echo '<a class="button-secondary" href="?page=feedbackcompany&legacywoocommerce">Legacy WooCommerce configuration</a> &nbsp; ';
		}

		echo '</div>';
	}

	/**
	 * Callback function for outputting a section
	 */
	public function section_output($section)
	{
		$tooltips = array(
			'section_wpml' => 'Selecting a specific language will cause the Feedback Company widgets and order registration to only be active for that language.',
			'section_oauth' => 'To get your client ID and client secret send an email to helpdesk@feedbackcompany.com',
			'section_woocommerce' => 'Configure which order status should trigger a review invitation and the delay before sending the review invitation and reminder. Also configure how to display the product reviews widget on the product page.',
			'section_productreviewswidget' => 'Configure how to display your product reviews widget.Visit https://feedbackcompany.wixanswers.com for more information.',
			'section_merchantreviewswidget' => "Configure how to display your merchant reviews widget. You can see a preview below. To show the widget, go to the 'Appearance - Widget' or 'Appearance - Customize' page and drag-and-drop the widget into the desired section, or use the shortcode in the text editor of a post or page",
			'section_widgetspreview' => 'Below is an example of your widgets using your selected configuration.',
			'section_productreview' => 'Here you can find the state and information of product review widgets',
		);

		echo '<span class="dashicons dashicons-editor-help" title="'.$tooltips[$section['id']].'"></span>';
		echo '<br style="clear: both;">';

		// oauth has notice added if succesfully connected
		if ($section['id'] == 'section_oauth')
		{
			if (feedbackcompany_api_wp()->ext->get_option('feedbackcompany_access_token'))
				echo '<span class="feedbackcompany-notice-ok">&#10004; Your Wordpress is connected to Feedback Company.</span>';
		}
		if ($section['id'] == 'section_productreview')
		{
			echo '<div style="float: left; margin-right: 20px;">';
			echo 'To enable your product reviews, please contact us at our email address <b>sales@feedbackcompany.com</b> or our phone number <b>+31 8 5273 6320</b>.';
			echo '<br>';
			echo 'You can check our page <a href="www.feedbackcompany.com/product-reviews">www.feedbackcompany.com/product-reviews</a> for more information about product reviews.';
			echo '</div>';
			echo '<br style="clear: both;">';
		}
		// widgetspreview is a special section with preview images
		if ($section['id'] == 'section_widgetspreview')
		{
			echo '<div style="float: left; margin-right: 20px;">';
			echo 'Merchant reviews<br><br>';
			echo '<img id="feedbackcompany_widgetpreview_merchantreviews">';
			echo '</div>';
			echo '<br style="clear: both;">';
		}
	}

	/**
	 * Callback functions for outputting specific form fields
	 *
	 * These functions and their $args are determined in this::page_init()
	 */
	public function field_output_orderstatus($args)
	{
		$options = wc_get_order_statuses();
		$this->field_output_selectbox($args, $options);
	}
	public function field_output_color($args)
	{
		$options = array('white' => 'white', 'blue' => 'blue');
		$this->field_output_selectbox($args, $options);
	}
	public function field_output_size($args)
	{
		$options = array('small' => 'small', 'big' => 'big');
		$this->field_output_selectbox($args, $options);
	}
	public function field_output_wpml($args)
	{
		$options = array('all' => 'All languages');
		$active_languages = apply_filters('wpml_active_languages', NULL);
		foreach ($active_languages as $language_key => $language_value)
		{
			$options[$language_key] = $language_value['translated_name'];
		}
		$this->field_output_selectbox($args, $options);
	}
	public function field_output_amount($args)
	{
		$options = array(0 => '0', 1 => '1', 2 => '2');
		$this->field_output_selectbox($args, $options);
	}
	public function field_output_displaytype($args)
	{
		$options = array('sidebar' => 'sidebar', 'popup' => 'popup', 'inline' => 'inline');
		$this->field_output_selectbox($args, $options);
	}
	public function field_output_boolean($args)
	{
		$options = array(1 => 'yes', 0 => 'no');
		$this->field_output_selectbox($args, $options);
	}
	public function field_output_shortcode($args)
	{
		echo '<input size="'.strlen($args['desc']).'" data-role="feedbackcompany-shortcode" '
			. 'id="shortcode_'.$args['name'].'" value="['.$args['desc'].']">';
		echo ' <a href="#" onclick="feedbackcompany_copytoclipboard(event, \'shortcode_'.$args['name'].'\');">copy</a>';
	}
	public function field_output_selectbox($args, $options)
	{
		echo '<select class="feedbackcompany-formfield" id="'.$args['name'].'" name="'.$args['name'].'">';
		foreach ($options as $value => $label)
			echo '<option value="'.$value.'" '
				. (esc_attr(get_option($args['name'])) == $value ? 'selected' : '')
				. '>'.esc_attr($label);
		echo '</select>';

		$this->field_output_description($args);
	}
	public function field_output_timeframe($args)
	{
		$options = array('minutes', 'hours', 'days', 'weekdays');
		echo '<input class="feedbackcompany-formfield" type="number" step="1" min="0" id="'.$args['name'].'" name="'.$args['name'].'" value="';
		echo esc_attr(get_option($args['name']));
		echo '"> ';

		echo '<select class="feedbackcompany-formfield" id="'.$args['name'].'_unit" name="'.$args['name'].'_unit">';
		foreach ($options as $option)
			echo '<option value="'.$option.'" '.(get_option($args['name'].'_unit') == $option ? 'selected' : '').'>'.$option;
		echo '</select>';

		$this->field_output_description($args);
	}
	public function field_output_text($args)
	{
		echo '<input class="feedbackcompany-formfield" type="text" id="'.$args['name'].'" name="'.$args['name'].'" size="70" value="';
		echo esc_attr(get_option($args['name']));
		echo '" />';

		$this->field_output_description($args);
	}
	/**
	 * helper function for description fields
	 *
	 * note: description fields are no longer used, except for displaying error messages
	 */
	public function field_output_description($args)
	{
		// if description is set
		if (isset($args['desc']))
			echo '<p class="description">'.esc_attr($args['desc']).'</p>';
	}

	/**
	 * Output function for legacy WooCommerce configuration screen
	 */
	public function output_adminpage_legacywoocommerce()
	{
		echo '<div class="wrap">';
		echo '<h1>Feedback Company Legacy Configuration - WooCommerce</h1>';
		echo '<div class="notice notice-warning"><p>'
			. '<strong>Warning!</strong>  The legacy WooCommerce connector is no longer developed and support will be dropped in a future version of this plugin. '
			. 'Please use the new version 2.3.1 order registration and product reviews whenever you can.'
			. '</p></div>';

		global $feedbackcompany_woocommerce;
		if ($feedbackcompany_woocommerce)
			$feedbackcompany_woocommerce->fc_settings_display();
		else
			echo 'Congratulations! The older, no longer supported WooCommerce connector is disabled on this Wordpress installation '
				. 'which means you are ready to use the shiny new order registration and product reviews widgets.'
				. '<br><br>'
				. '<a class="button-primary" href="?page=feedbackcompany">Click here to go the configuration of the current WooCommerce extension</a>';
		echo '</div>';
	}

	/**
	 * Output function for legacy widgets configuration screen
	 */
	public function output_adminpage_legacywidgets()
	{
		echo '<div class="wrap">';
		echo '<h1>Feedback Company Legacy Configuration - Review Widgets</h1>';
		echo '<div class="notice notice-warning"><p>'
			. '<strong>Warning!</strong>  Legacy widgets are no longer developed and support will be dropped in a future version of this plugin. '
			. 'Please use the new version 2.3.1 widgets whenever you can.'
			. '</p></div>';

		global $feedbackcompany_legacywidget_adminsettings;
		if ($feedbackcompany_legacywidget_adminsettings)
			$feedbackcompany_legacywidget_adminsettings->output_adminpage();
		else
			echo 'Congratulations! The older, no longer supported legacy widgets are disabled on this Wordpress installation '
				. 'which means you are ready to use the shiny new widgets.'
				. '<br><br>'
				. '<a class="button-primary" href="?page=feedbackcompany">Click here to go the configuration of the current widgets</a>';
		echo '</div>';
	}

	/**
	 * Functions for handling updated settings
	 *
	 * there are actions registered at the bottom of this file, to add actions if above settings are changed
	 * these actions don't register new widgets directly, because for each individual widget setting there is a seperate action
	 * to prevent any unnecessary calls, the action-functions set variables inside this object
	 * and on shutdown the 'updatedsettings' function is called, to perform the actual actions if settings have changed
	 *
	 * note that these functions are called on 'shutdown', rather than via the destructor, to ensure PHP 5.x compatibility
	 */
	public function updated_oauth($old_value, $new_value)
	{
		$this->updated_oauth = true;
	}
	public function updated_widgetmain($old_value, $new_value)
	{
		$this->updated_widgetmain = true;
	}
	public function updated_widgetproductextended($old_value, $new_value)
	{
		$this->updated_widgetproductextended = true;
	}

	/**
	 * This function is called on shutdown, to perform updates if needed
	 * determined by if action variables are set to true
	 */
	public function updatedsettings()
	{
		// check if product reviews are enabled for loading the product widgets
		$before_product_reviews_enabled_check = get_option('feedbackcompany_productreviews_enabled');
		feedbackcompany_api_wp()->check_product_reviews_enabled();

		if ($before_product_reviews_enabled_check <> get_option('feedbackcompany_productreviews_enabled'))
		{
			$this->updated_widgetproductsummary = true;
			$this->updated_widgetproductextended = true;
		}

		// if oauth settings are updated, force new widgets to be registered
		$force_refresh = false;

		// actions to perform if certain settings have been updated
		if ($this->updated_oauth)
		{
			// clear the cache and access token
			feedbackcompany_api_wp()->clear_cache();

			// request an access token
			feedbackcompany_api_wp()->oauth_refreshtoken();

			// if access token was not successfully retrieved, stop here
			if (!feedbackcompany_api_wp()->ext->get_option('feedbackcompany_access_token'))
				return;

			$force_refresh = true;
			$this->updated_widgetmain = true;
			$this->updated_widgetproductsummary = true;
			$this->updated_widgetproductextended = true;
		}

		if ($this->updated_widgetmain)
		{
			feedbackcompany_api_wp()->register_widget_main(
				get_option('feedbackcompany_merchantreviewswidget_size'),
				get_option('feedbackcompany_merchantreviewswidget_amount'),
				$force_refresh
			);
		}

		if ($this->updated_widgetproductsummary && get_option('feedbackcompany_productreviews_enabled'))
		{
			feedbackcompany_api_wp()->register_widget_productsummary($force_refresh);
		}

		if ($this->updated_widgetproductextended && get_option('feedbackcompany_productreviews_enabled'))
		{
			feedbackcompany_api_wp()->register_widget_productextended(
				get_option('feedbackcompany_productreviewsextendedwidget_displaytype'),
				$force_refresh
			);
		}

	}
}

// initialize the class above
$feedbackcompany_admin_settings_page = new feedbackcompany_admin_settings();

// register actions to request new widget uuids on changing of settings
add_action('update_option_feedbackcompany_oauth_client_id', array($feedbackcompany_admin_settings_page, 'updated_oauth'), 10, 2);
add_action('update_option_feedbackcompany_oauth_client_secret', array($feedbackcompany_admin_settings_page, 'updated_oauth'), 10, 2);
add_action('update_option_feedbackcompany_merchantreviewswidget_size', array($feedbackcompany_admin_settings_page, 'updated_widgetmain'), 10, 2);
add_action('update_option_feedbackcompany_merchantreviewswidget_amount', array($feedbackcompany_admin_settings_page, 'updated_widgetmain'), 10, 2);
add_action('update_option_feedbackcompany_productreviewsextendedwidget_displaytype', array($feedbackcompany_admin_settings_page, 'updated_widgetproductextended'), 10, 2);
add_action('update_option_feedbackcompany_productreviewsextendedwidget_toggle_element', array($feedbackcompany_admin_settings_page, 'updated_widget_toggle_element'), 10, 2);
// register action to update widget settings on shutdown
add_action('shutdown', array($feedbackcompany_admin_settings_page, 'updatedsettings'), 10, 2);
